(function ($) {
  Drupal.cacheControl = {};
  Drupal.cacheControl.initialized = false;

  Drupal.cacheControl.componentHandlers = [];
  Drupal.cacheControl.componentsFinishedHandlers = [];

  Drupal.behaviors.cacheControl = {
    attach: function(context, settings) {
      if (Drupal.cacheControl.initialized) {
        //There's no need to ever execute this more than once per pageload
        cache_control_debug('cc already initialized');
        return;
      }
      Drupal.cacheControl.initialized = true;
      cache_control_debug('cc active');
      if (settings.cacheControl && (settings.cacheControl.cachePageComponents || settings.cacheControl.cacheNodeId)) {
        cache_control_debug('cc running');
        var cacheNodeId = settings.cacheControl.cacheNodeId;

        var force = settings.cacheControl.force || false;
        var authenticated = parseInt($.cookie('cacheControlAuthenticated'), 10);
        cache_control_debug(authenticated);
        if (authenticated || force) {
          cache_control_debug('cc getting components');

          var componentIds = [];
          var componentHandlers = Drupal.cacheControl.componentHandlers;

          for (var i = 0; i < settings.cacheControl.cachePageComponents.length; i++) {
            var component = settings.cacheControl.cachePageComponents[i];

            //See if everyone is ok with this component
            var fetchComponent = true;
            for (var j = 0; j < componentHandlers.length; j++) {
              var handler = componentHandlers[j];
              if (typeof handler == 'function') {
                if (handler(component) === false) {
                  fetchComponent = false;
                  //Use anonymous content instead
                  var anonymousElem = $("#" + component.html_id + '-anonymous');
                  var placeholderElem = $("#" + component.html_id);
                  if (anonymousElem.length > 0) {
                	if (placeholderElem.length > 0) {
                      placeholderElem.remove();
                	}
                    anonymousElem.attr('id', component.html_id);
                    anonymousElem.removeClass('cacheControlAnonymous');
                  }
                }
              }
            }

            if (fetchComponent) {
              componentIds.push(component.id);
            }
          }

          if (componentIds.length > 0) {
            var path = settings.basePath
              + "_cache_control/get_components/u/"
              + authenticated
              + "/"
              + $.cookie('cacheControlSessionHash')
              + "/"
              + componentIds.join(',');

            $.getJSON(path, function(data) {
              //Inject components into DOM
              cache_control_debug(data);

          	  if (data.settings) {
          		//If some components delivered settings, take them into use.
          	    jQuery.extend(Drupal.settings, data.settings);
          	  }

              for (var i in data.components) if (data.components.hasOwnProperty(i)) {
                cache_control_debug('cc handling element ' + i);
                var elem = $(data.components[i]);
                cache_control_debug(elem);
                var id = elem.attr("id");
                cache_control_debug(id);
                //Remove the anonymous content to avoid conflicts
                var anonymousElement = $("#" + id + "-anonymous");
                var placeholderElement = $("#" + id);
                Drupal.detachBehaviors(anonymousElement);

                if (placeholderElement.length > 0) {
                  anonymousElement.remove();
                  Drupal.detachBehaviors(placeholderElement);
                  placeholderElement.replaceWith(elem);
                }
                else {
                  anonymousElement.replaceWith(elem);
                }

                //Some JS components may need initializing
                for (var b in Drupal.behaviors) if (Drupal.behaviors.hasOwnProperty(b)) {
                  if (b !== 'cacheControl' && typeof Drupal.behaviors[b].attach === 'function') {
                    Drupal.behaviors[b].attach(elem, Drupal.settings);
                  }
                }
              }

              //Give modules/themes (usually themes) a chance to react to the fact that the page is now completely ready.
              var finishHandlers = Drupal.cacheControl.componentsFinishedHandlers;
              for (var i = 0; i < finishHandlers.length; i++) {
                var handler = finishHandlers[i];
                if (typeof handler == 'function') {
                  handler();
                }
              }
            });
          }
        }

        if (!authenticated) {
          //Bring up anonymous content
          $(".cacheControlAnonymous").each(function() {
            var id = $(this).attr('id').replace(/-anonymous$/, '');

            if ($("#" + id).length > 0) {
              $("#" + id).replaceWith($(this));
            }
            $(this).attr('id', id);
            $(this).removeClass('cacheControlAnonymous');

          });

          //If needed, process forms with captcha
          if (settings.cacheControl.processAnonymousCaptchaForms) {
            $("input[name=captcha_response]").each(function() {
              var form = $(this).parents('form');
              if (!form.hasClass('cacheControlProcessedForm')) {
                form.addClass('cacheControlProcessedForm');
                var formBuildId = form.find('input[name=form_build_id]').val();

                //Add a focus event handler that silently and hopefully unobtrusively rebuilds the form in the background
                form.delegate('*', 'focus', function() {
                  //store focused element id so that we can restore focus when the form has been re-rendered
                  var focusedId = $(this).attr("id");

                  $.get(Drupal.settings.basePath + "_cache_control/rebuild_form/" + formBuildId, function(data) {
                    var currentContent = $("#" + focusedId).val();
                    var newForm = $(data);
                    form.replaceWith(newForm);

                    //Some JS components may need initializing
                    for (var b in Drupal.behaviors) if (Drupal.behaviors.hasOwnProperty(b)) {
                      if (b !== 'cacheControl' && typeof Drupal.behaviors[b].attach === 'function') {
                        Drupal.behaviors[b].attach(newForm, Drupal.settings);
                      }
                    }

                    //For improved user experience
                    $("#" + focusedId)
                      .val(currentContent)
                      .focus();
                  });
                });
              }
            });
          }

          //Further, track statistics for an anonymous user if so requested
          if (cacheNodeId) {
            $.get(Drupal.settings.basePath + "cache_control_statistics_tracker.php?nid=" + cacheNodeId);
          }
        }
      }
    }
  };

  function cache_control_debug(str) {
    // console.log(str);
  }
})(jQuery);
